@@ -1,55 +1,75 @@
-/* evilwm - Minimalist Window Manager for X
- * Copyright (C) 1999-2015 Ciaran Anscomb <evilwm@6809.org.uk>
+/* evilwm - minimalist window manager for X11
+ * Copyright (C) 1999-2021 Ciaran Anscomb <evilwm@6809.org.uk>
  * see README for license and other details. */
 
+// X11 event processing.  This is the core of the window manager and processes
+// events from clients and user interaction in a loop until signalled to exit.
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdlib.h>
 #include <string.h>
-#include <errno.h>
-#include <sys/select.h>
+
+#include <X11/X.h>
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
+
 #include <X11/XKBlib.h>
-#include "evilwm.h"
-#include "log.h"
+#include <X11/keysymdef.h>
 
-static int interruptibleXNextEvent(XEvent *event);
-
-#ifdef DEBUG
-const char *debug_atom_name(Atom a);
-const char *debug_atom_name(Atom a) {
-	static char buf[48];
-	char *atom_name = XGetAtomName(dpy, a);
-	strncpy(buf, atom_name, sizeof(buf));
-	buf[sizeof(buf)-1] = 0;
-	return buf;
-}
+#ifdef SHAPE
+#include <X11/extensions/shape.h>
+#endif
+#ifdef RANDR
+#include <X11/extensions/Xrandr.h>
 #endif
 
-static void handle_key_event(XKeyEvent *e) {
-	KeySym key = XkbKeycodeToKeysym(dpy, e->keycode, 0, 0);
-	Client *c;
-	int width_inc, height_inc;
-	ScreenInfo *current_screen = find_current_screen();
+#include "client.h"
+#include "display.h"
+#include "events.h"
+#include "evilwm.h"
+#include "ewmh.h"
+#include "keymap.h"
+#include "list.h"
+#include "log.h"
+#include "screen.h"
+#include "util.h"
 
-	switch(key) {
+// Event loop will run until this flag is set
+int wm_exit;
+
+// Flags that the client list should be scanned and marked clients removed.
+// Set by unhandled X errors and unmap requests.
+int need_client_tidy = 0;
+
+// Process keyboard events.
+
+static void handle_key_event(XKeyEvent *e) {
+	KeySym key = XkbKeycodeToKeysym(display.dpy, e->keycode, 0, 0);
+	struct screen *current_screen = find_current_screen();
+
+	switch (key) {
 		case KEY_NEW:
-			spawn((const char *const *)opt_term);
+			spawn((const char *const *)option.term);
 			break;
 		case KEY_NEXT:
-			next();
-			if (XGrabKeyboard(dpy, e->root, False, GrabModeAsync, GrabModeAsync, CurrentTime) == GrabSuccess) {
+			client_select_next();
+			if (XGrabKeyboard(display.dpy, e->root, False, GrabModeAsync, GrabModeAsync, CurrentTime) == GrabSuccess) {
 				XEvent ev;
 				do {
-					XMaskEvent(dpy, KeyPressMask|KeyReleaseMask, &ev);
-					if (ev.type == KeyPress && XkbKeycodeToKeysym(dpy, ev.xkey.keycode, 0, 0) == KEY_NEXT)
-						next();
-				} while (ev.type == KeyPress || XkbKeycodeToKeysym(dpy, ev.xkey.keycode, 0, 0) == KEY_NEXT);
-				XUngrabKeyboard(dpy, CurrentTime);
+					XMaskEvent(display.dpy, KeyPressMask|KeyReleaseMask, &ev);
+					if (ev.type == KeyPress && XkbKeycodeToKeysym(display.dpy, ev.xkey.keycode, 0, 0) == KEY_NEXT)
+						client_select_next();
+				} while (ev.type == KeyPress || XkbKeycodeToKeysym(display.dpy, ev.xkey.keycode, 0, 0) == KEY_NEXT);
+				XUngrabKeyboard(display.dpy, CurrentTime);
 			}
-			ewmh_select_client(current);
+			clients_tab_order = list_to_head(clients_tab_order, current);
 			break;
 		case KEY_DOCK_TOGGLE:
 			set_docks_visible(current_screen, !current_screen->docks_visible);
 			break;
-#ifdef VWM
 		case XK_1: case XK_2: case XK_3: case XK_4:
 		case XK_5: case XK_6: case XK_7: case XK_8:
 			switch_vdesk(current_screen, KEY_TO_VDESK(key));
@@ -69,13 +89,17 @@ static void handle_key_event(XKeyEvent *e) {
 		case KEY_TOGGLEDESK:
 			switch_vdesk(current_screen, current_screen->old_vdesk);
 			break;
-#endif
-		default: break;
+		default:
+			break;
 	}
-	c = current;
+
+	struct client *c = current;
 	if (c == NULL) return;
-	width_inc = (c->width_inc > 1) ? c->width_inc : 16;
-	height_inc = (c->height_inc > 1) ? c->height_inc : 16;
+
+	struct monitor *monitor = client_monitor(c, NULL);
+	int width_inc = (c->width_inc > 1) ? c->width_inc : 16;
+	int height_inc = (c->height_inc > 1) ? c->height_inc : 16;
+
 	switch (key) {
 		case KEY_LEFT:
 			if (e->state & altmask) {
@@ -110,24 +134,20 @@ static void handle_key_event(XKeyEvent *e) {
 			}
 			goto move_client;
 		case KEY_TOPLEFT:
-			c->x = c->border;
-			c->y = c->border;
+			c->x = monitor->x + c->border;
+			c->y = monitor->y + c->border;
 			goto move_client;
 		case KEY_TOPRIGHT:
-			c->x = DisplayWidth(dpy, c->screen->screen)
-				- c->width-c->border;
-			c->y = c->border;
+			c->x = monitor->x + monitor->width - c->width-c->border;
+			c->y = monitor->y + c->border;
 			goto move_client;
 		case KEY_BOTTOMLEFT:
-			c->x = c->border;
-			c->y = DisplayHeight(dpy, c->screen->screen)
-				- c->height-c->border;
+			c->x = monitor->x + c->border;
+			c->y = monitor->y + monitor->height - c->height-c->border;
 			goto move_client;
 		case KEY_BOTTOMRIGHT:
-			c->x = DisplayWidth(dpy, c->screen->screen)
-				- c->width-c->border;
-			c->y = DisplayHeight(dpy, c->screen->screen)
-				- c->height-c->border;
+			c->x = monitor->x + monitor->width - c->width-c->border;
+			c->y = monitor->y + monitor->height - c->height-c->border;
 			goto move_client;
 		case KEY_KILL:
 			send_wm_delete(c, e->state & altmask);
@@ -136,15 +156,18 @@ static void handle_key_event(XKeyEvent *e) {
 			client_lower(c);
 			break;
 		case KEY_INFO:
-			show_info(c, e->keycode);
+			client_show_info(c, e->keycode);
 			break;
 		case KEY_MAX:
-			maximise_client(c, NET_WM_STATE_TOGGLE, MAXIMISE_HORZ|MAXIMISE_VERT);
+			client_maximise(c, NET_WM_STATE_TOGGLE, MAXIMISE_HORZ|MAXIMISE_VERT);
 			break;
 		case KEY_MAXVERT:
-			maximise_client(c, NET_WM_STATE_TOGGLE, MAXIMISE_VERT);
+			if (e->state & altmask) {
+				client_maximise(c, NET_WM_STATE_TOGGLE, MAXIMISE_HORZ);
+			} else {
+				client_maximise(c, NET_WM_STATE_TOGGLE, MAXIMISE_VERT);
+			}
 			break;
-#ifdef VWM
 		case KEY_FIX:
 			if (is_fixed(c)) {
 				client_to_vdesk(c, current_screen->vdesk);
@@ -152,53 +175,69 @@ static void handle_key_event(XKeyEvent *e) {
 				client_to_vdesk(c, VDESK_FIXED);
 			}
 			break;
-#endif
-		default: break;
+		default:
+			break;
 	}
 	return;
+
 move_client:
 	if (abs(c->x) == c->border && c->oldw != 0)
 		c->x = 0;
 	if (abs(c->y) == c->border && c->oldh != 0)
 		c->y = 0;
-	moveresize(c);
+	client_moveresizeraise(c);
 #ifdef WARP_POINTER
-	setmouse(c->window, c->width + c->border - 1,
-			c->height + c->border - 1);
+	setmouse(c->window, c->width + c->border - 1, c->height + c->border - 1);
 #endif
 	discard_enter_events(c);
 	return;
 }
 
+// Handle mousebutton events.
+
 static void handle_button_event(XButtonEvent *e) {
-	Client *c = find_client(e->window);
+	struct client *c = find_client(e->window);
 
 	if (c) {
 		switch (e->button) {
 			case Button1:
-				drag(c); break;
+				client_move_drag(c, e->button);
+				break;
 			case Button2:
-				sweep(c); break;
+				client_resize_sweep(c, e->button);
+				break;
 			case Button3:
-				client_lower(c); break;
-			default: break;
+				client_lower(c);
+				break;
+			default:
+				break;
 		}
 	}
 }
 
-static void do_window_changes(int value_mask, XWindowChanges *wc, Client *c,
+// Apply the changes from an XWindowChanges struct to a client.
+
+static void do_window_changes(int value_mask, XWindowChanges *wc, struct client *c,
 		int gravity) {
+	LOG_XENTER("do_window_changes(window=%lx), was: %dx%d+%d+%d", (unsigned long)c->window, c->width, c->height, c->x, c->y);
 	if (gravity == 0)
 		gravity = c->win_gravity_hint;
 	c->win_gravity = gravity;
-	if (value_mask & CWX) c->x = wc->x;
-	if (value_mask & CWY) c->y = wc->y;
+	if (value_mask & CWX) {
+		c->x = wc->x;
+		LOG_XDEBUG("CWX      x=%d\n", wc->x);
+	}
+	if (value_mask & CWY) {
+		c->y = wc->y;
+		LOG_XDEBUG("CWY      y=%d\n", wc->y);
+	}
 	if (value_mask & (CWWidth|CWHeight)) {
 		int dw = 0, dh = 0;
 		if (!(value_mask & (CWX|CWY))) {
-			gravitate_border(c, -c->border);
+			client_gravitate(c, -c->border);
 		}
 		if (value_mask & CWWidth) {
+			LOG_XDEBUG("CWWidth  width=%d\n", wc->width);
 			int neww = wc->width;
 			if (neww < c->min_width)
 				neww = c->min_width;
@@ -208,6 +247,7 @@ static void do_window_changes(int value_mask, XWindowChanges *wc, Client *c,
 			c->width = neww;
 		}
 		if (value_mask & CWHeight) {
+			LOG_XDEBUG("CWHeight height=%d\n", wc->height);
 			int newh = wc->height;
 			if (newh < c->min_height)
 				newh = c->min_height;
@@ -216,7 +256,8 @@ static void do_window_changes(int value_mask, XWindowChanges *wc, Client *c,
 			dh = newh - c->height;
 			c->height = newh;
 		}
-		/* only apply position fixes if not being explicitly moved */
+
+		// only apply position fixes if not being explicitly moved
 		if (!(value_mask & (CWX|CWY))) {
 			switch (gravity) {
 			default:
@@ -252,23 +293,25 @@ static void do_window_changes(int value_mask, XWindowChanges *wc, Client *c,
 				break;
 			}
 			value_mask |= CWX|CWY;
-			gravitate_border(c, c->border);
+			client_gravitate(c, c->border);
 		}
 	}
+
 	wc->x = c->x - c->border;
 	wc->y = c->y - c->border;
 	wc->border_width = c->border;
-	XConfigureWindow(dpy, c->parent, value_mask, wc);
-	XMoveResizeWindow(dpy, c->window, 0, 0, c->width, c->height);
+	XConfigureWindow(display.dpy, c->parent, value_mask, wc);
+	XMoveResizeWindow(display.dpy, c->window, 0, 0, c->width, c->height);
 	if ((value_mask & (CWX|CWY)) && !(value_mask & (CWWidth|CWHeight))) {
 		send_config(c);
 	}
+	LOG_XLEAVE();
 }
 
 static void handle_configure_request(XConfigureRequestEvent *e) {
-	Client *c = find_client(e->window);
-	XWindowChanges wc;
+	struct client *c = find_client(e->window);
 
+	XWindowChanges wc;
 	wc.x = e->x;
 	wc.y = e->y;
 	wc.width = e->width;
@@ -276,9 +319,10 @@ static void handle_configure_request(XConfigureRequestEvent *e) {
 	wc.border_width = 0;
 	wc.sibling = e->above;
 	wc.stack_mode = e->detail;
+
 	if (c) {
 		if (e->value_mask & CWStackMode && e->value_mask & CWSibling) {
-			Client *sibling = find_client(e->above);
+			struct client *sibling = find_client(e->above);
 			if (sibling) {
 				wc.sibling = sibling->parent;
 			}
@@ -288,35 +332,33 @@ static void handle_configure_request(XConfigureRequestEvent *e) {
 			discard_enter_events(c);
 		}
 	} else {
-		LOG_XENTER("XConfigureWindow(window=%lx, value_mask=%lx)", (unsigned int)e->window, e->value_mask);
-		XConfigureWindow(dpy, e->window, e->value_mask, &wc);
+		LOG_XENTER("XConfigureWindow(window=%lx, value_mask=%lx)", (unsigned long)e->window, e->value_mask);
+		XConfigureWindow(display.dpy, e->window, e->value_mask, &wc);
 		LOG_XLEAVE();
 	}
 }
 
 static void handle_map_request(XMapRequestEvent *e) {
-	Client *c = find_client(e->window);
+	struct client *c = find_client(e->window);
 
-	LOG_ENTER("handle_map_request(window=%lx)", e->window);
+	LOG_ENTER("handle_map_request(window=%lx)", (unsigned long)e->window);
 	if (c) {
-#ifdef VWM
 		if (!is_fixed(c) && c->vdesk != c->screen->vdesk)
 			switch_vdesk(c->screen, c->vdesk);
-#endif
 		client_show(c);
 		client_raise(c);
 	} else {
 		XWindowAttributes attr;
-		XGetWindowAttributes(dpy, e->window, &attr);
-		make_new_client(e->window, find_screen(attr.root));
+		XGetWindowAttributes(display.dpy, e->window, &attr);
+		client_manage_new(e->window, find_screen(attr.root));
 	}
 	LOG_LEAVE();
 }
 
 static void handle_unmap_event(XUnmapEvent *e) {
-	Client *c = find_client(e->window);
+	struct client *c = find_client(e->window);
 
-	LOG_ENTER("handle_unmap_event(window=%lx)", e->window);
+	LOG_ENTER("handle_unmap_event(window=%lx)", (unsigned long)e->window);
 	if (c) {
 		if (c->ignore_unmap) {
 			c->ignore_unmap--;
@@ -333,29 +375,25 @@ static void handle_unmap_event(XUnmapEvent *e) {
 }
 
 static void handle_colormap_change(XColormapEvent *e) {
-	Client *c = find_client(e->window);
+	struct client *c = find_client(e->window);
 
 	if (c && e->new) {
 		c->cmap = e->colormap;
-		XInstallColormap(dpy, c->cmap);
+		XInstallColormap(display.dpy, c->cmap);
 	}
 }
 
 static void handle_property_change(XPropertyEvent *e) {
-	Client *c = find_client(e->window);
+	struct client *c = find_client(e->window);
 
 	if (c) {
-		LOG_ENTER("handle_property_change(window=%lx, atom=%s)", e->window, debug_atom_name(e->atom));
+		LOG_ENTER("handle_property_change(window=%lx, atom=%s)", (unsigned long)e->window, debug_atom_name(e->atom));
 		if (e->atom == XA_WM_NORMAL_HINTS) {
 			get_wm_normal_hints(c);
 			LOG_DEBUG("geometry=%dx%d+%d+%d\n", c->width, c->height, c->x, c->y);
-		} else if (e->atom == xa_net_wm_window_type) {
+		} else if (e->atom == X_ATOM(_NET_WM_WINDOW_TYPE)) {
 			get_window_type(c);
-			if (!c->is_dock
-#ifdef VWM
-					&& (is_fixed(c) || (c->vdesk == c->screen->vdesk))
-#endif
-					) {
+			if (!c->is_dock && (is_fixed(c) || (c->vdesk == c->screen->vdesk))) {
 				client_show(c);
 			}
 		}
@@ -364,15 +402,13 @@ static void handle_property_change(XPropertyEvent *e) {
 }
 
 static void handle_enter_event(XCrossingEvent *e) {
-	Client *c;
+	struct client *c;
 
 	if ((c = find_client(e->window))) {
-#ifdef VWM
 		if (!is_fixed(c) && c->vdesk != c->screen->vdesk)
 			return;
-#endif
 		select_client(c);
-		ewmh_select_client(c);
+		clients_tab_order = list_to_head(clients_tab_order, c);
 	}
 }
 
@@ -380,66 +416,94 @@ static void handle_mappingnotify_event(XMappingEvent *e) {
 	XRefreshKeyboardMapping(e);
 	if (e->request == MappingKeyboard) {
 		int i;
-		for (i = 0; i < num_screens; i++) {
-			grab_keys_for_screen(&screens[i]);
+		for (i = 0; i < display.nscreens; i++) {
+			grab_keys_for_screen(&display.screens[i]);
 		}
 	}
 }
 
 #ifdef SHAPE
 static void handle_shape_event(XShapeEvent *e) {
-	Client *c = find_client(e->window);
+	struct client *c = find_client(e->window);
 	if (c)
 		set_shape(c);
 }
 #endif
 
-static void handle_client_message(XClientMessageEvent *e) {
-#ifdef VWM
-	ScreenInfo *s = find_current_screen();
+#ifdef RANDR
+static void handle_randr_event(XRRScreenChangeNotifyEvent *e) {
+	struct screen *s = find_screen(e->root);
+	// Record geometries of clients relative to monitor
+	scan_clients_before_resize(s);
+	// Update Xlib's idea of screen size
+	XRRUpdateConfiguration((XEvent*)e);
+	// Scan new monitor list
+	screen_probe_monitors(s);
+	// Fix any clients that are now not visible on any monitor.  Also
+	// adjusts maximised geometries where appropriate.
+	fix_screen_after_resize(s);
+	// Update various EWMH properties that reflect screen geometry
+	ewmh_set_screen_workarea(s);
+}
 #endif
-	Client *c;
 
-	LOG_ENTER("handle_client_message(window=%lx, format=%d, type=%s)", e->window, e->format, debug_atom_name(e->message_type));
+// Events sent to clients
 
-#ifdef VWM
-	if (e->message_type == xa_net_current_desktop) {
+static void handle_client_message(XClientMessageEvent *e) {
+	struct screen *s = find_current_screen();
+	struct client *c;
+
+	LOG_ENTER("handle_client_message(window=%lx, format=%d, type=%s)", (unsigned long)e->window, e->format, debug_atom_name(e->message_type));
+
+	if (e->message_type == X_ATOM(_NET_CURRENT_DESKTOP)) {
 		switch_vdesk(s, e->data.l[0]);
 		LOG_LEAVE();
 		return;
 	}
-#endif
+
 	c = find_client(e->window);
-	if (!c && e->message_type == xa_net_request_frame_extents) {
-		ewmh_set_net_frame_extents(e->window);
-		LOG_LEAVE();
-		return;
-	}
 	if (!c) {
+
+		// _NET_REQUEST_FRAME_EXTENTS is intended to be sent from
+		// unmapped windows.  The reply is just an _estimate_ of the
+		// border widths, so for the moment, just guess that it'll be
+		// the default (it may not be if MWM flags suggest no border).
+		//
+		// XXX: would a client expect a reply if this was sent while
+		// mapped?
+		//
+		// TODO: calculate this according to the logic in
+		// client_manage_new() instead of offering up this "estimate".
+
+		if (e->message_type == X_ATOM(_NET_REQUEST_FRAME_EXTENTS)) {
+			ewmh_set_net_frame_extents(e->window, option.bw);
+		}
+
 		LOG_LEAVE();
 		return;
 	}
-	if (e->message_type == xa_net_active_window) {
-		/* Only do this if it came from direct user action */
+
+	if (e->message_type == X_ATOM(_NET_ACTIVE_WINDOW)) {
+		// Only do this if it came from direct user action
 		if (e->data.l[0] == 2) {
-#ifdef VWM
 			if (c->screen == s)
-#endif
 				select_client(c);
 		}
 		LOG_LEAVE();
 		return;
 	}
-	if (e->message_type == xa_net_close_window) {
-		/* Only do this if it came from direct user action */
+
+	if (e->message_type == X_ATOM(_NET_CLOSE_WINDOW)) {
+		// Only do this if it came from direct user action
 		if (e->data.l[1] == 2) {
 			send_wm_delete(c, 0);
 		}
 		LOG_LEAVE();
 		return;
 	}
-	if (e->message_type == xa_net_moveresize_window) {
-		/* Only do this if it came from direct user action */
+
+	if (e->message_type == X_ATOM(_NET_MOVERESIZE_WINDOW)) {
+		// Only do this if it came from direct user action
 		int source_indication = (e->data.l[0] >> 12) & 3;
 		if (source_indication == 2) {
 			int value_mask = (e->data.l[0] >> 8) & 0x0f;
@@ -455,8 +519,9 @@ static void handle_client_message(XClientMessageEvent *e) {
 		LOG_LEAVE();
 		return;
 	}
-	if (e->message_type == xa_net_restack_window) {
-		/* Only do this if it came from direct user action */
+
+	if (e->message_type == X_ATOM(_NET_RESTACK_WINDOW)) {
+		// Only do this if it came from direct user action
 		if (e->data.l[0] == 2) {
 			XWindowChanges wc;
 
@@ -467,87 +532,110 @@ static void handle_client_message(XClientMessageEvent *e) {
 		LOG_LEAVE();
 		return;
 	}
-#ifdef VWM
-	if (e->message_type == xa_net_wm_desktop) {
-		/* Only do this if it came from direct user action */
+
+	if (e->message_type == X_ATOM(_NET_WM_DESKTOP)) {
+		// Only do this if it came from direct user action
 		if (e->data.l[1] == 2) {
 			client_to_vdesk(c, e->data.l[0]);
 		}
 		LOG_LEAVE();
 		return;
 	}
-#endif
-	if (e->message_type == xa_net_wm_state) {
+
+	if (e->message_type == X_ATOM(_NET_WM_STATE)) {
 		int i, maximise_hv = 0;
-		/* Message can contain up to two state changes: */
+		// Message can contain up to two state changes:
 		for (i = 1; i <= 2; i++) {
-			if ((Atom)e->data.l[i] == xa_net_wm_state_maximized_vert) {
+			if ((Atom)e->data.l[i] == X_ATOM(_NET_WM_STATE_MAXIMIZED_VERT)) {
 				maximise_hv |= MAXIMISE_VERT;
-			} else if ((Atom)e->data.l[i] == xa_net_wm_state_maximized_horz) {
+			} else if ((Atom)e->data.l[i] == X_ATOM(_NET_WM_STATE_MAXIMIZED_HORZ)) {
 				maximise_hv |= MAXIMISE_HORZ;
-			} else if ((Atom)e->data.l[i] == xa_net_wm_state_fullscreen) {
+			} else if ((Atom)e->data.l[i] == X_ATOM(_NET_WM_STATE_FULLSCREEN)) {
 				maximise_hv |= MAXIMISE_VERT|MAXIMISE_HORZ;
 			}
 		}
 		if (maximise_hv) {
-			maximise_client(c, e->data.l[0], maximise_hv);
+			client_maximise(c, e->data.l[0], maximise_hv);
 		}
 		LOG_LEAVE();
 		return;
 	}
+
 	LOG_LEAVE();
 }
 
+// Run the main event loop.  This will run until something tells us to quit
+// (generally, a signal).
+
 void event_main_loop(void) {
+	// XEvent is a big union of all the core event types, but we also need
+	// to handle events about extensions, so make a union of the union...
 	union {
 		XEvent xevent;
 #ifdef SHAPE
 		XShapeEvent xshape;
+#endif
+#ifdef RANDR
+		XRRScreenChangeNotifyEvent xrandr;
 #endif
 	} ev;
-	/* main event loop here */
+
+	// Main event loop
 	while (!wm_exit) {
 		if (interruptibleXNextEvent(&ev.xevent)) {
 			switch (ev.xevent.type) {
 			case KeyPress:
-				handle_key_event(&ev.xevent.xkey); break;
+				handle_key_event(&ev.xevent.xkey);
+				break;
 			case ButtonPress:
-				handle_button_event(&ev.xevent.xbutton); break;
+				handle_button_event(&ev.xevent.xbutton);
+				break;
 			case ConfigureRequest:
-				handle_configure_request(&ev.xevent.xconfigurerequest); break;
+				handle_configure_request(&ev.xevent.xconfigurerequest);
+				break;
 			case MapRequest:
-				handle_map_request(&ev.xevent.xmaprequest); break;
+				handle_map_request(&ev.xevent.xmaprequest);
+				break;
 			case ColormapNotify:
-				handle_colormap_change(&ev.xevent.xcolormap); break;
+				handle_colormap_change(&ev.xevent.xcolormap);
+				break;
 			case EnterNotify:
-				handle_enter_event(&ev.xevent.xcrossing); break;
+				handle_enter_event(&ev.xevent.xcrossing);
+				break;
 			case PropertyNotify:
-				handle_property_change(&ev.xevent.xproperty); break;
+				handle_property_change(&ev.xevent.xproperty);
+				break;
 			case UnmapNotify:
-				handle_unmap_event(&ev.xevent.xunmap); break;
+				handle_unmap_event(&ev.xevent.xunmap);
+				break;
 			case MappingNotify:
-				handle_mappingnotify_event(&ev.xevent.xmapping); break;
+				handle_mappingnotify_event(&ev.xevent.xmapping);
+				break;
 			case ClientMessage:
-				handle_client_message(&ev.xevent.xclient); break;
+				handle_client_message(&ev.xevent.xclient);
+				break;
 			default:
 #ifdef SHAPE
-				if (have_shape && ev.xevent.type == shape_event) {
+				if (display.have_shape
+				    && ev.xevent.type == display.shape_event) {
 					handle_shape_event(&ev.xshape);
 				}
 #endif
 #ifdef RANDR
-				if (have_randr && ev.xevent.type == randr_event_base + RRScreenChangeNotify) {
-					XRRUpdateConfiguration(&ev.xevent);
+				if (display.have_randr && ev.xevent.type == display.randr_event_base + RRScreenChangeNotify) {
+					handle_randr_event(&ev.xrandr);
 				}
 #endif
 				break;
 			}
 		}
+
+		// Scan list for clients flagged to be removed
 		if (need_client_tidy) {
 			struct list *iter, *niter;
 			need_client_tidy = 0;
 			for (iter = clients_tab_order; iter; iter = niter) {
-				Client *c = iter->data;
+				struct client *c = iter->data;
 				niter = iter->next;
 				if (c->remove)
 					remove_client(c);
@@ -555,37 +643,3 @@ void event_main_loop(void) {
 		}
 	}
 }
-
-/* interruptibleXNextEvent() is taken from the Blender source and comes with
- * the following copyright notice: */
-
-/* Copyright (c) Mark J. Kilgard, 1994, 1995, 1996. */
-
-/* This program is freely distributable without licensing fees
- * and is provided without guarantee or warrantee expressed or
- * implied. This program is -not- in the public domain. */
-
-/* Unlike XNextEvent, if a signal arrives, interruptibleXNextEvent will
- * return zero. */
-
-static int interruptibleXNextEvent(XEvent *event) {
-	fd_set fds;
-	int rc;
-	int dpy_fd = ConnectionNumber(dpy);
-	for (;;) {
-		if (XPending(dpy)) {
-			XNextEvent(dpy, event);
-			return 1;
-		}
-		FD_ZERO(&fds);
-		FD_SET(dpy_fd, &fds);
-		rc = select(dpy_fd + 1, &fds, NULL, NULL, NULL);
-		if (rc < 0) {
-			if (errno == EINTR) {
-				return 0;
-			} else {
-				LOG_ERROR("interruptibleXNextEvent(): select()\n");
-			}
-		}
-	}
-}
