@@ -3,7 +3,11 @@
 
 #define KEY_NEXT        XK_Tab
 #define KEY_NEW         XK_Return
-#define KEY_TOPLEFT     XK_y
+#ifndef QWERTZ_KEYMAP
+# define KEY_TOPLEFT    XK_y
+#else
+# define KEY_TOPLEFT    XK_z
+#endif
 #define KEY_TOPRIGHT    XK_u
 #define KEY_BOTTOMLEFT  XK_b
 #define KEY_BOTTOMRIGHT XK_n
@@ -17,12 +21,10 @@
 #define KEY_MAXVERT     XK_equal
 #define KEY_MAX         XK_x
 #define KEY_DOCK_TOGGLE XK_d
-#ifdef VWM
-# define KEY_FIX        XK_f
-# define KEY_PREVDESK   XK_Left
-# define KEY_NEXTDESK   XK_Right
-# define KEY_TOGGLEDESK XK_a
-#endif
+#define KEY_FIX        XK_f
+#define KEY_PREVDESK   XK_Left
+#define KEY_NEXTDESK   XK_Right
+#define KEY_TOGGLEDESK XK_a
 /* Mixtures of Ctrl, Alt an Escape are used for things like VMWare and
  * XFree86/Cygwin, so the KILL key is an option in the Makefile  */
 #ifndef KEY_KILL
